#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os, logging
import sys
from collections import defaultdict

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsWNDMerger2(MergerInterface):
  """!
  Class for merging synsets graph and WND graph.

  WND graph was built based on the WND graph file, which is described in 
  WNDGraphBuilder class. In WND graph each node represents one domain/concept. 
  Whereas each edge represents relation occurring between them.

  The purpose of this class is to add edges from WND graph to synsets graph. 
  New edges between synsets are added according to relation between WND edges. 
  Thus, each edge from WND graph is inserted to the synsets graph.
  """
  def __init__(self, resources, options, str_name = 'SynsetsWNDMerger2'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsWNDMerger"
    """
    super(SynsetsWNDMerger2, self).__init__(resources, options, str_name)

  def get_plwn2wnd_dict(self):
    """!
    Create dictionary based on mapping PLWN on WND file. Dictionary format 
    and mapping PLWN on WND file format are presented below.

    Format of mapping PLWN on WND file:
    \code
    synset_str_plwn;domain_plwn;synset_id_plwn;interlingual_rel; \
    synset_str_pwn;pwn_synset_pos;domain_pwn;synset_id_pwn;wnd_domain
    \endcode

    Sample format of mapping PLWN on WND file:
    \code
    adres.5;por-7;16;syn_pa; \
    address.6;n;por-7;321457;geography

    adwokat.1;os-15;17;syncz_pa; \
    advocate.2;n;os-15;337414;law

    ...
    \endcode

    Format of dictionary:
    \code
    wnd_concept -> set([synset_id_1, ..., synset_id_N])
    \endcode

    Sample format of created dictionary:
    \code
    geography -> set([19247])
    law -> set([25849, 25851, 44443, 398925])
    ...
    \endcode

    @return dictionary, where key is a WND concept and value is a set of synsets 
            identifiers
    """
    if not os.path.exists(self.resources().mapping_wnd_file()):
      raise IOError(
        "%s file not found!" % \
          self.resources().mapping_wnd_file()
          )

    plwn2wnd_dict = defaultdict(set)

    with open(self.resources().mapping_wnd_file()) as wndfile:
      next(wndfile)
      for line in wndfile:
        synset_id = int(line.strip().split(';')[2])
        wnd = line.strip().split(';')[8]
        plwn2wnd_dict[wnd].add(synset_id)

    return plwn2wnd_dict

  def merge(self, g1, g2):
    """!
    Merge two given graphs, namely synsets graph and WND graph. The final graph contain 
    one type of nodes, namely synsets nodes. Each synset node has an attribute named "synset", 
    to which is assigned synset object. Synset object consist of: synset_id and lu_set, 
    which contains lexical units objects. Whereas lexical units objects consists of: lu_id, 
    lemma, pos, variant and domain. The final graph contain two types of relation. 
    First type is relation occurring between synsets nodes only, which have assigned 
    relation identifier from PLWN. Second type is relation occurring between 
    WND concepts, but proper synsets nodes are connected. If two WND concepts are
    connected in WND graph and they have mapping on some synsets then synsets nodes 
    will be connected. This connection has two attributes: "rel" and "rel_type". 
    "rel" is a relation name from WND graph, whereas to "rel_type" is assigned word "wnd".

    The directness of the graph and uniqueness of edges are inherited from synsets graph.

    @param g1 - synsets graph
    @param g2 - WND graph
    @return object of BaseGraph class
    """
    logger = logging.getLogger(__name__)
    
    
    g = BaseGraph()
    g.copy_graph_from(g1)

    plwn2wnd_dict = defaultdict(set)
    plwn2wnd_dict = self.get_plwn2wnd_dict()

    synset_on_vertex_dict = {}
    for node in g.all_nodes():
      synset_id = node.synset.synset_id
      if synset_id in synset_on_vertex_dict:
        logger.warning("ID of some synset is not unique.")
        continue
      synset_on_vertex_dict[synset_id] = node

    num_of_edge = 0
    for edge in g2.all_edges():
      num_of_edge += 1
      logger.info('%d/%d', num_of_edge, g2.num_edges())

      parent_wnd_concept = edge.source().wnd
      child_wnd_concept = edge.target().wnd

      if parent_wnd_concept not in plwn2wnd_dict:
        logger.warning("The mapping file doesn't contain wnd '%s'.", parent_wnd_concept)
        continue
      if child_wnd_concept not in plwn2wnd_dict:
        logger.warning("The mapping file doesn't contain wnd '%s'.", child_wnd_concept)
        continue

      for parent_syn_id in plwn2wnd_dict[parent_wnd_concept]:
        if parent_syn_id not in synset_on_vertex_dict:
          logger.warning("The mapping file contains synset '%d' that is not in the graph.", parent_syn_id)
          continue
        p_node = synset_on_vertex_dict[parent_syn_id]
        for child_syn_id in plwn2wnd_dict[child_wnd_concept]:
          if child_syn_id not in synset_on_vertex_dict:
            logger.warning("The mapping file contains synset '%d' that is not in the graph.", child_syn_id)
            continue
          ch_node = synset_on_vertex_dict[child_syn_id]
          g.add_edge(p_node,
                     ch_node,
                     [("rel", edge.rel)],
                     simply=True)

    return g



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTPersonalizedPRNormTwoStep(WSDAlgorithmInterface):
  """!
  Two-step Word Sense Disambiguation.

  The disambiguation process was divided into two steps with running Page Rank
  into both.

  This method used one big graph, which are build by merging e.g. SUMO graph and
  plWordNet graph.

  In the first step (called as coarse grained disambiguation), e.g. SUMO ontology
  is utilised as a network of interconnected concepts. For each word "w" from
  the context, we choose the set of concepts connected with "w" and initialise
  personalized vector "v". Then, the process of coarse disambiguation is run.
  After the disambiguation, for word "w" we choose only one, the best concept.

  The second phase of disambiguation (fine-grained disambiguation) based on
  plWordNet graph and synset mappings onto SUMO ontology. The difference is
  in "v" initialisation. Only these elements are initialised, which has
  connections (mappings) with concept chosen in previous step. For example,
  for word "zamek", in the previous stage the "Lock" concept has been assigned,
  only two from six synsets are initialised: {zamek 2 'lock' (wytw)} and
  {zamek 5 'semaphore' (wytw)}. Next, the disambiguation process is run
  and as the result the ranking value for each node is returned. For each word,
  the synset with the highest ranking value is chosen as a meaning of words.

  Initialization process of personalized vector "v" was described below.

  If lemma has N meanings (synsets/concepts) then values in personalized vector 
  "v" for all its meanings (synsets/concepts) will be 1/N. If this same synset
  occur in two lemmas from the analyzed context then boths values are added.

  For example:
  \code
  Lemma    Set of synsets for lemma
  kot      1, 2, 3, 4, 5
  pies     5, 6, 7

  Personalized vector "v":
  Synset ID             Value
  1 (kot-1)             1/5
  2 (kot-2)             1/5
  3 (kot-3)             1/5
  4 (kot-4)             1/5
  5 (kot-5) & (pies-1)  1/5 + 1/3
  6 (pies-2)            1/3
  7 (pies-3)            1/3
  \endcode

  Remaining synsets which do not belong to lemmas from the analyzed context
  have value equal to 0 in personalized vector "v".
  """

  def __init__(self, str_name = 'GTPersonalizedPRNormTwoStep'):
    """!
    @param str_name - default value same as the class name "GTPersonalizedPRNormTwoStep"
    """
    super(GTPersonalizedPRNormTwoStep, self).__init__(str_name)
    self._context_node_dict = None

  def prepare_v(self, wsd_context, graph):
    """!
    Create personalized vector "v".

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    v = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_dict:
      v[node.use_graph_tool()] = self._multiply_factor * self._context_node_dict[node]
    return v
  
  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())

    # Pierwszy etap ujednoznaczniania:
    # Wazne jest aby w pliku konfiguracyjnym ustawic parametr ini_nodes = sumo.
    self._set_context_node_dict_for_first_step(lemma_on_node_dict)

    pers_v = self.prepare_v(wsd_context, graph)
    (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = pers_v, 
                                   max_iter = 2 * options.max_iter(),
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])

    best_concept_for_lemma_dict = {}
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      if (lemma, pos_str) in lemma_on_node_dict:
        nodes = lemma_on_node_dict[(lemma, pos_str)]
        best_node = None
        best_ranking = 0.0
        for node in nodes:
          if best_ranking < ranking[node.use_graph_tool()]:
            best_node = node
            best_ranking = ranking[node.use_graph_tool()]
        best_concept_for_lemma_dict[(lemma, pos_str)] = best_node
      else:
        best_concept_for_lemma_dict[(lemma, pos_str)] = None

    # Drugi etap ujednoznaczniania:
    self._set_context_node_dict_for_second_step(best_concept_for_lemma_dict,
                                                lemma_on_only_synset_node_dict,
                                                graph)

    pers_v = self.prepare_v(wsd_context, graph)
    (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = pers_v, 
                                   max_iter = 2 * options.max_iter(),
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])
    
    ranking = graph.ungraph_tool(ranking)
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, ranking)

    return (wsd_rank, ret_iter)

  def _set_context_node_dict(self, nodes_set):
    """!
    Create a dictionary where key is object of node from the graph which
    corespond to the synset which belong to lemma from the analyzed context.
    Whereas value is a value which synset from the key will be have in
    personalized vector "v".

    @param nodes_set - set of nodes from the graph
    """
    nodes_set_size = float(len(nodes_set))
    for node in nodes_set:
      if self._context_node_dict.has_key(node):
        self._context_node_dict[node] += 1.0 / nodes_set_size
      else:
        self._context_node_dict[node] = 1.0 / nodes_set_size

  def _set_context_node_dict_for_first_step(self, lemma_on_node_dict):
    """!
    Create a dictionary where key is object of node from the graph which
    corespond to the synset which belong to lemma from the analyzed context.
    Whereas value is a value which synset from the key will be have in
    personalized vector "v" for the first step of disambiguation process.

    @param lemma_on_node_dict - dictionary where key is (lemma, pos_str) and
                                value is set of synsets (nodes from the graph)
                                belong to this lemma
    """
    self._context_node_dict = {}
    for nodes in lemma_on_node_dict.itervalues():
      self._set_context_node_dict(nodes)

  def _set_context_node_dict_for_second_step(self, 
                                             best_concept_for_lemma_dict,
                                             lemma_on_only_synset_node_dict,
                                             graph):
    """!
    Create a dictionary where key is object of node from the graph which
    corespond to the synset which belong to lemma from the analyzed context.
    Whereas value is a value which synset from the key will be have in
    personalized vector "v" for the second step of disambiguation process.

    @param best_concept_for_lemma_dict - dictionary where key is (lemma, pos_str)
                                         and value is node which corespond to the
                                         best concept chosen in the first stage
                                         of word sense disambiguation
    @param lemma_on_only_synset_node_dict - dictionary where key is (lemma, pos_str)
                                            and value is set of synsets (nodes
                                            from the graph) belong to this lemma
    """
    self._context_node_dict = {}
    for (lemma, pos_str), node in best_concept_for_lemma_dict.iteritems():
      synsets_nodes_set = set()
      synsets_nodes_set = lemma_on_only_synset_node_dict[(lemma, pos_str)]
      if node:
        chosen_synsets_nodes_set = set()
        for ngb in node.all_neighbours():
          if ngb.synset:
            if ngb in synsets_nodes_set:
              chosen_synsets_nodes_set.add(ngb)
        self._set_context_node_dict(chosen_synsets_nodes_set)
      else:
        self._set_context_node_dict(synsets_nodes_set)



from abc import abstractmethod



class LeskFunctionInterface(object):
  """! Interface that shall be implemented by all functions of Lesk. """
  
  def __init__(self, resources, filter_obj, graph, options, str_name):  #TODO co znaczy "paragraph object from Corpus2"?
    """!
    Standard constructor.
    
    @type tagset: corpus2.Tagset
    @param tagset: tagset
    
    @type filter_obj: Filter
    @param filter_obj: filter of unwanted words
    
    @type graph: graph_tool.Graph
    @param graph: graph
    
    @type gloss_dict: dict([int, paragraph object from Corpus2])
    @param gloss_dict: definitions for synsets
    
    @type options: AlgorithmOptions
    @param options: options for algorithms
    
    @type str_name: str
    @param str_name: name to be displayed, it should be name of class
    """
    self.options = options
    self.tagset = resources.tagset()
    self.graph = graph
    self.gloss_dict = resources.gloss_file()
    self.filter_obj = filter_obj
    self._str_name = str_name

  def __str__(self):
    return self._str_name

  def prepare_node(self, node):
    """!
    Prepare any data you like. It will be called at most once for each node.
    
    @type node: graph_tool.Vertex
    @param node: node from graph
    
    @rtype: object
    @return: whatever you like
    """
    return None
  
  def prepare_context(self, context):
    """!
    Prepare any data you like. It will be called at most once for each context.
    
    @type context: WSDContext
    @param context: context to be processed
    
    @rtype: object
    @return: whatever you like
    """
    return None
  
  @abstractmethod
  def compare(self, node_data, context_data):
    """!
    Check how well given definition matches context.
    
    @type node_data: object
    @param node_data: data created in prepare_node()
    
    @type context_data: object
    @param context_data: data created in prepare_context()
    
    @rtype: float
    @return: how likely it is that this definition be proper one
    """
    raise NotImplementedError("LeskFunctionInterface!")







# -*- coding: utf-8 -*-

import unittest
from wosedon.lesk_functions.cosine import _cosine_similarity, Cosine
from wosedon.utils.config import Config





class TestCosineSimilarity(unittest.TestCase):

  def test_cosine_function(self):
    self.assertAlmostEquals(_cosine_similarity(
                      [2, 24, 3, 64, -2, -5], [2, 24, 3, 64, -2, -5]), 1)
    self.assertAlmostEquals(_cosine_similarity(
                      [-1, 2], [1, 2]), 0.6)
    self.assertAlmostEquals(_cosine_similarity(
                      [17, 213, 33, 11, 3453], [17, 34, 11, 33, 456]), 0.996929679772)
    self.assertAlmostEquals(_cosine_similarity(
                      [3, -2], [2, 3]), 0)
    self.assertAlmostEquals(_cosine_similarity(
                      [1, 2, 3, 4, 5, 6, 7], [4, 2, 4, 5, 6, 2, 1]), 0.744776393793)
    self.assertAlmostEquals(_cosine_similarity(
                      [1, 2, 3], [0, 0, 0]), 0)
    




class TestLeskCosine(unittest.TestCase):

  def setUp(self):
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_cosine.ini')
    
    self.graph = cfg.gbuilders()[0].build_graph()
    self.context = cfg.context('tests/#Data/wiki_ccl.xml', 'tests/#Data/wiki_ccl.xml')
    self.cosine = cfg.algorithm_options().lesk_function(cfg.resources(), self.graph)
  
  
  def test_cosine(self):
    
    definition_data = {
      ('aaaa', 'adv') : 1,
      ('bbbb', 'subst') : 2,
      ('ccccc', 'fin') : 3,
      ('ddd', 'adja') : 4,
      ('eee', 'depr') : 5,
      ('fff', 'inf') : 6,
      ('ggg', 'pact') : 7
    }
    
    context_data = {
      ('aaaa', 'adv') : 4,
      ('bbbb', 'subst') : 2,
      ('ccccc', 'fin') : 4,
      ('ddd', 'adja') : 5,
      ('eee', 'depr') : 6,
      ('fff', 'inf') : 2,
      ('ggg', 'pact') : 1,
      ('afsadfs', 'inf') : 1352352,
      ('hthhj', 'inf') : 356456464,
      ('ewrwre', 'inf') : 2352343
    }
    
    self.assertAlmostEquals(self.cosine.compare(definition_data, context_data), 0.744776393793)
  
  
  def test_run_prepare_node(self):
    self.graph.create_edge_attribute('weight', 'float')
    for node in self.graph.all_nodes():
      res = self.cosine.prepare_node(node)
      self.assertIsInstance(res, dict)
      for k in res:
        self.assertIsInstance(k, tuple)
        self.assertIsInstance(k[0], str)
        self.assertIsInstance(k[1], str)
        self.assertIsInstance(res[k], float)
  
  
  def test_run_prepare_context(self):
    sen = self.context.next()
    while sen:
      res = self.cosine.prepare_context(sen)
      self.assertIsInstance(res, dict)
      for k in res:
        self.assertIsInstance(k, tuple)
        self.assertIsInstance(k[0], str)
        self.assertIsInstance(k[1], str)
        self.assertIsInstance(res[k], float)
      sen = self.context.next()












from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface

class POS(LeskFilterInterface):
  """! Allows only some Parts of Speech """
  
  def __init__(self, tagset, options, str_name='POS'):
    super(POS, self).__init__(tagset, options, str_name)
    
    self._allow_only = options.allow_only()
    self._set = set()
    
    if options.input_file():
      with open(options.input_file()) as f:
        for line in f:
          line = line.strip()
          self._set.add(line)
      
  
  def __contains__(self, lexeme):
    pos = self.tagset.get_pos_name(lexeme.tag().get_pos_index())
    return self._allow_only == (pos in self._set)






#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os, sys, logging
from collections import defaultdict

from wosedon.basegraph import BaseGraph
from graphbuilderinterface import GraphBuilderInterface

class BidirectionalMSRGraphBuilder(GraphBuilderInterface):
  """!
  Class for building graph from k-best file.

  The built graph is based on the k-best file. K-best file contain words, where 
  for each word there are a list of k most relatedness word to it. To each word 
  in the list of k most relatedness words is also assigned the value of the 
  relatedness measure. K-best file is created based on the corpus and on 
  Measures of Semantic Relatedness.

  The difference between BidirectionalMSRGraphBuilder class and MSRGraphBuilder 
  class is that the BidirectionalMSRGraphBuilder class take into account only 
  symmetric (bidirectional) relations between words.

  Sample format of k-best file:
  \code
  subst:akcyza
    subst:podatek akcyzowy;0.491485
    subst:podatek;0.360689
    subst:agresja;0.235534
  subst:agresja
    subst:przemoc;0.334895
    subst:agresywnosc;0.327364
    subst:akcyza;0.235534
  ...
  \endcode
  """

  def __init__(self, resources, options, 
               str_name = 'BidirectionalMSRGraphBuilder'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of BuildOptions class contain build options from 
                       config file
    @param str_name  - default value same as the class name 
                       "BidirectionalMSRGraphBuilder"
    """
    super(BidirectionalMSRGraphBuilder, self).__init__(
      resources, options, str_name)
    self._kbests_dict = None

  def build_graph(self):
    """!
    Build graph based on _kbests_dict. Each node has an attribute named "msr" 
    which contain word and POS written as word/POS, e.g. akcyza/2. If word does not 
    have coarse POS, then will be ignored. To each edge is assigned weight, 
    which is the value of measure of semantic relatedness between linked words.

    The graph will be always undirected. Whereas all edges duplicates is removed.

    @return object of BaseGraph class
    """
    logger = logging.getLogger(__name__)
    
    
    if not os.path.exists(self.resources().msr_file()):
      raise IOError(
        "%s file not found!" % \
          self.resources().msr_file())

    self._read_kbests()
    self._bidirectional_kbest()

    g = BaseGraph()
    g.init_graph(drctd = False)

    g.create_node_attribute("msr", "string")
    g.create_edge_attributes([("weight", "double"), ('rel', 'string')])

    for parent, childs in self._kbests_dict.iteritems():
      lemma = parent.strip().split('/')[0]
      pl_pos = parent.strip().split('/')[1]
      pos_int = self._convert_to_coarse_pos(pl_pos)
      if not pos_int:
        logger.warning('Unknown POS "%s" for lemma "%s"', pl_pos, lemma)
        continue
      parent = lemma + '/' + str(pos_int)
      g.add_node(parent, [("msr", parent)])
      for (child, weight) in childs:
        lemma = child.strip().split('/')[0]
        pl_pos = child.strip().split('/')[1]
        pos_int = self._convert_to_coarse_pos(pl_pos)
        if not pos_int:
          logger.warning('Unknown POS "%s" for lemma "%s"', pl_pos, lemma)
          continue
        child = lemma + '/' + str(pos_int)
        g.add_node(child, [("msr", child)])
        if parent != child:
          g.add_edge(parent, child, [("weight", weight)])
  
    g.remove_edge_duplicates()

    return g

  def _read_kbests(self):
    """!
    Read k-best file and create a _kbest_dict dictionary in the format describe below. 
    The key in dictionary is a main word with part of speech (POS), where the 
    value is a set of tuples (word/POS, value of measure) from relatedness list.

    Sample format of created dictionary:
    \code
    akcyza/subst -> set([(podatek akcyzowy/subst, 0.491485), (podatek/subst, 0.360689), (agresja/subst, 0.235534)])
    agresja/subst -> set([(przemoc/subst, 0.334895), (agresywnosc/subst, 0.327364), (akcyza/subst, 0.235534)])
    ...
    \endcode
    """
    key = None
    self._kbests_dict = defaultdict(set)
    with open(self.resources().msr_file()) as msrgfile:
      for line in msrgfile:
        if not line.startswith('\t'):
          pl_pos = line.strip().split(':')[0]
          lemma = line.strip().split(':')[1]
          key = lemma + '/' + pl_pos
        else:
          pl_pos = line.strip().split(';')[0].split(':')[0]
          lemma = line.strip().split(';')[0].split(':')[1]
          weight = float(line.strip().split(';')[1])
          self._kbests_dict[key].add((lemma + '/' + pl_pos, weight))

  def _bidirectional_kbest(self):
    """!
    This function checks whether the word which is on the relatedness list of 
    the main word has also the main word on it's relatedness list. Thus, 
    whether the relation is symmetric or not. It should be noted, that parts of 
    speech are take into account during the checking process. If the word does not 
    have a main word on relatedness list it is removed from dictionary from main 
    word set. The content of dictionary will change from this given in _read_kbests 
    function to the:
    \code
    akcyza/subst -> set([(agresja/subst, 0.235534)])
    agresja/subst -> set([(akcyza/subst, 0.235534)])
    ...
    \endcode
    """
    # lista rodzicow do usuniecia, ktorzy nie maja dzieci
    parents_to_remove_list = []
    
    for parent, childs in self._kbests_dict.iteritems():
      # lista dzieci do usuniecia, 
      # ktore nie maja rodzicow jako kbest w swoim zbiorze
      childs_to_remove_list = []
      
      for (child, weight) in childs:
        if self._kbests_dict.has_key(child):
          if (parent, weight) in self._kbests_dict[child]:
            continue
          else:
            childs_to_remove_list.append((child, weight))
        else:
          childs_to_remove_list.append((child, weight))
      for (child, weight) in childs_to_remove_list:
        childs.remove((child, weight))
      if not childs:
        parents_to_remove_list.append(parent)
    for parent in parents_to_remove_list:
      del self._kbests_dict[parent]

  def _convert_to_coarse_pos(self, pl_pos):
    """!
    For a given fine POS return coarse POS in integer type.

    @param pl_pos - fine part of speech
    @return integer coarse part of speech where 1:verb, 2:noun, 3:adverb and 4:adjective
    """
    #! List of tags which can describe nouns.
    noun_pl_pos = ['subst', 'depr', 'ger', 'brev']
    #! List of tags which can describe adjectives.
    adj_pl_pos  = ['adj', 'adja', 'adjp', 'adjc']
    #! List of tags which can describe verbs.
    verb_pl_pos = ['fin', 'bedzie', 'praet', 'impt', \
                   'inf', 'pcon', 'pant', 'imps', \
                   'winien', 'pred', 'pact', 'ppas', 'pred']
    #! List of tags which can describe adverbs.
    adv_pl_pos  = ['adv']

    pos_int = None
    if pl_pos in noun_pl_pos:
      pos_int = 2
    elif pl_pos in adj_pl_pos:
      pos_int = 4
    elif pl_pos in verb_pl_pos:
      pos_int = 1
    elif pl_pos in adv_pl_pos:
      pos_int = 3
    return pos_int



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

class BuildOptions:
  def __init__(self):
    self._accept_lexicon = []
    self._unique_edges = True
    self._directed_graph = True
    self._syn_rel_ids = []
    self._lu_rel_ids = []
    self._accept_pos = []
    self._add_reversed_edges = {}

  def set_accept_lexicon(self, alexicon):
    self._accept_lexicon = alexicon

  def set_unique_edges(self, uqe):
    self._unique_edges = uqe

  def set_directed_graph(self, directed):
    self._directed_graph = directed
  
  def set_syn_rel_ids(self, sri):
    self._syn_rel_ids = sri
  
  def set_lu_rel_ids(self, lri):
    self._lu_rel_ids = lri
  
  def set_accept_pos(self, apos):
    self._accept_pos = apos

  def set_add_reversed_edges(self, are):
    self._add_reversed_edges = are

  def accept_lexicon(self):
    return self._accept_lexicon

  def has_unique_edges(self):
    return self._unique_edges

  def is_directed_graph(self):
    return self._directed_graph
  
  def syn_rel_ids(self):
    return self._syn_rel_ids
    
  def lu_rel_ids(self):
    return self._lu_rel_ids
  
  def accept_pos(self):
    return self._accept_pos
  
  def get_add_reversed_edges(self):
    return self._add_reversed_edges





from wndgraphbuilder import WNDGraphBuilder
from synsetgraphbuilder import SynsetGraphBuilder
from sumographbuilder import SUMOGraphBuilder
from msrgraphbuilder import MSRGraphBuilder
from lexicalunitgraphbuilder import LexicalUnitGraphBuilder
from glossgraphbuilder import GlossGraphBuilder
from bidirectionalmsrgraphbuilder import BidirectionalMSRGraphBuilder
from cclgraphbuilder import CCLGraphBuilder
